@@ -15,16 +15,19 @@
 #' `list_sizes()` returns an integer vector containing the size of each element
 #' of a list. It is nearly equivalent to, but faster than,
 #' `map_int(x, vec_size)`, with the exception that `list_sizes()` will
-#' error on non-list inputs, as defined by [vec_is_list()]. `list_sizes()` is
+#' error on non-list inputs, as defined by [obj_is_list()]. `list_sizes()` is
 #' to `vec_size()` as [lengths()] is to [length()].
 #'
 #' @seealso [vec_slice()] for a variation of `[` compatible with `vec_size()`,
-#'   and [vec_recycle()] to recycle vectors to common length.
+#'   and [vec_recycle()] to [recycle][theory-faq-recycling] vectors to common
+#'   length.
 #' @section Invariants:
 #' * `vec_size(dataframe)` == `vec_size(dataframe[[i]])`
 #' * `vec_size(matrix)` == `vec_size(matrix[, i, drop = FALSE])`
 #' * `vec_size(vec_c(x, y))` == `vec_size(x)` + `vec_size(y)`
 #'
+#' @inheritParams rlang::args_error_context
+#'
 #' @param x,... Vector inputs or `NULL`.
 #' @param .size If `NULL`, the default, the output size is determined by
 #'   recycling the lengths of all elements of `...`. Alternatively, you can
@@ -79,19 +82,23 @@
 #'
 #' list_sizes(list("a", 1:5, letters))
 vec_size <- function(x) {
-  .Call(vctrs_size, x)
+  .Call(ffi_size, x, environment())
 }
 
 #' @export
 #' @rdname vec_size
-vec_size_common <- function(..., .size = NULL, .absent = 0L) {
-  .External2(vctrs_size_common, .size, .absent)
+vec_size_common <- function(...,
+                            .size = NULL,
+                            .absent = 0L,
+                            .arg = "",
+                            .call = caller_env()) {
+  .External2(ffi_size_common, .size, .absent)
 }
 
 #' @rdname vec_size
 #' @export
 list_sizes <- function(x) {
-  .Call(vctrs_list_sizes, x)
+  .Call(ffi_list_sizes, x, environment())
 }
 
 #' @rdname vec_size
@@ -144,3 +151,9 @@ vec_seq_along <- function(x) {
 vec_init_along <- function(x, y = x) {
   vec_slice(x, rep_len(NA_integer_, vec_size(y)))
 }
+
+vec_as_short_length <- function(n,
+                                arg = caller_arg(n),
+                                call = caller_env()) {
+  .Call(ffi_as_short_length, n, environment())
+}
