@@ -40,22 +40,22 @@ df_as_tibble <- function(df) {
 
 # Conditionally registered in .onLoad()
 
-vec_ptype2.tbl_df.tbl_df <- function(x, y, ...) {
+vec_ptype2_tbl_df_tbl_df <- function(x, y, ...) {
   vec_ptype2_dispatch_native(x, y, ...)
 }
-vec_ptype2.tbl_df.data.frame <- function(x, y, ...) {
+vec_ptype2_tbl_df_data.frame <- function(x, y, ...) {
   vec_ptype2_dispatch_native(x, y, ...)
 }
-vec_ptype2.data.frame.tbl_df <- function(x, y, ...) {
+vec_ptype2_data.frame_tbl_df <- function(x, y, ...) {
   vec_ptype2_dispatch_native(x, y, ...)
 }
 
-vec_cast.tbl_df.tbl_df <- function(x, to, ...) {
+vec_cast_tbl_df_tbl_df <- function(x, to, ...) {
   vec_cast_dispatch_native(x, to, ...)
 }
-vec_cast.data.frame.tbl_df <- function(x, to, ...) {
+vec_cast_data.frame_tbl_df <- function(x, to, ...) {
   vec_cast_dispatch_native(x, to, ...)
 }
-vec_cast.tbl_df.data.frame <- function(x, to, ...) {
+vec_cast_tbl_df_data.frame <- function(x, to, ...) {
   vec_cast_dispatch_native(x, to, ...)
 }
