@@ -8,6 +8,11 @@
 
 void vctrs_init_altrep_rle(DllInfo* dll) { }
 
+SEXP altrep_rle_is_materialized(SEXP x) {
+  Rf_error("Need R 3.5+ for Altrep support.");
+  return R_NilValue;
+}
+
 SEXP altrep_rle_Make(SEXP input) {
   Rf_error("Need R 3.5+ for Altrep support.");
   return R_NilValue;
@@ -19,6 +24,9 @@ SEXP altrep_rle_Make(SEXP input) {
 // Initialised at load time
 R_altrep_class_t altrep_rle_class;
 
+SEXP altrep_rle_is_materialized(SEXP x) {
+  return Rf_ScalarLogical(R_altrep_data2(x) != R_NilValue);
+}
 
 SEXP altrep_rle_Make(SEXP input) {
   SEXP res = R_new_altrep(altrep_rle_class, input, R_NilValue);
@@ -51,7 +59,7 @@ Rboolean altrep_rle_Inspect(SEXP x,
                             int deep,
                             int pvec,
                             void (*inspect_subtree)(SEXP, int, int, int)) {
-  Rprintf("vctrs_altrep_rle (len=%d, materialized=%s)\n",
+  Rprintf("vctrs_altrep_rle (len=%" R_PRIdXLEN_T ", materialized=%s)\n",
           altrep_rle_Length(x),
           R_altrep_data2(x) != R_NilValue ? "T" : "F");
   return TRUE;
