@@ -13,7 +13,7 @@
 #ifndef VCTRS_ORDER_SORTEDNESS_H
 #define VCTRS_ORDER_SORTEDNESS_H
 
-#include "vctrs.h"
+#include "vctrs-core.h"
 #include "order-groups.h"
 
 // -----------------------------------------------------------------------------
@@ -30,6 +30,7 @@ enum vctrs_sortedness dbl_sortedness(const double* p_x,
                                      r_ssize size,
                                      bool decreasing,
                                      bool na_last,
+                                     bool nan_distinct,
                                      struct group_infos* p_group_infos);
 
 enum vctrs_sortedness int_sortedness(const int* p_x,
