@@ -137,7 +137,7 @@ SEXP chr_resize(SEXP x, r_ssize x_size, r_ssize size);
 SEXP s3_get_method(const char* generic, const char* cls, SEXP table);
 SEXP s3_sym_get_method(SEXP sym, SEXP table);
 SEXP s3_find_method(const char* generic, SEXP x, SEXP table);
-SEXP s3_class_find_method(const char* generic, SEXP class, SEXP table);
+SEXP s3_class_find_method(const char* generic, SEXP cls, SEXP table);
 SEXP s3_get_class(SEXP x);
 SEXP s3_find_method_xy(const char* generic,
                        SEXP x,
@@ -151,7 +151,7 @@ SEXP s3_find_method2(const char* generic,
 SEXP s3_paste_method_sym(const char* generic, const char* cls);
 SEXP s3_bare_class(SEXP x);
 SEXP s4_find_method(SEXP x, SEXP table);
-SEXP s4_class_find_method(SEXP class, SEXP table);
+SEXP s4_class_find_method(SEXP cls, SEXP table);
 bool vec_implements_ptype2(SEXP x);
 
 SEXP r_env_get(SEXP env, SEXP sym);
@@ -166,7 +166,7 @@ SEXP list_first_non_null(SEXP xs, R_len_t* non_null_i);
 bool list_is_homogeneously_classed(SEXP xs);
 
 // Destructive compacting
-SEXP node_compact_d(SEXP xs);
+SEXP node_compact_d(SEXP node);
 
 void never_reached(const char* fn) __attribute__((noreturn));
 
@@ -256,13 +256,6 @@ SEXP r_new_list(R_len_t n) {
   return Rf_allocVector(VECSXP, n);
 }
 
-static inline
-SEXP r_new_environment(SEXP parent) {
-  SEXP env = Rf_allocSExp(ENVSXP);
-  SET_ENCLOS(env, parent);
-  return env;
-}
-
 SEXP r_protect(SEXP x);
 bool r_is_number(SEXP x);
 bool r_is_positive_number(SEXP x);
@@ -445,8 +438,10 @@ extern SEXP chrs_asc;
 extern SEXP chrs_desc;
 extern SEXP chrs_largest;
 extern SEXP chrs_smallest;
+extern SEXP chrs_which;
 
 extern SEXP syms_i;
+extern SEXP syms_j;
 extern SEXP syms_n;
 extern SEXP syms_x;
 extern SEXP syms_y;
@@ -491,11 +486,16 @@ extern SEXP syms_s3_fallback;
 extern SEXP syms_stop_incompatible_type;
 extern SEXP syms_stop_incompatible_size;
 extern SEXP syms_stop_assert_size;
+extern SEXP syms_stop_matches_overflow;
 extern SEXP syms_stop_matches_nothing;
 extern SEXP syms_stop_matches_remaining;
 extern SEXP syms_stop_matches_incomplete;
 extern SEXP syms_stop_matches_multiple;
 extern SEXP syms_warn_matches_multiple;
+extern SEXP syms_stop_matches_relationship_one_to_one;
+extern SEXP syms_stop_matches_relationship_one_to_many;
+extern SEXP syms_stop_matches_relationship_many_to_one;
+extern SEXP syms_warn_matches_relationship_many_to_many;
 extern SEXP syms_action;
 extern SEXP syms_vctrs_common_class_fallback;
 extern SEXP syms_fallback_class;
@@ -506,6 +506,7 @@ extern SEXP syms_actual;
 extern SEXP syms_required;
 extern SEXP syms_call;
 extern SEXP syms_dot_call;
+extern SEXP syms_which;
 
 static const char * const c_strs_vctrs_common_class_fallback = "vctrs:::common_class_fallback";
 
