@@ -2,8 +2,12 @@
 # Rational record class from the S3 vector vignette
 
 new_rational <- function(n = integer(), d = integer()) {
-  vec_assert(n, ptype = integer())
-  vec_assert(d, ptype = integer())
+  if (!is_integer(n)) {
+    abort("`n` must be an integer.")
+  }
+  if (!is_integer(d)) {
+    abort("`d` must be an integer.")
+  }
   new_rcrd(list(n = n, d = d), class = "vctrs_rational")
 }
 
@@ -32,7 +36,7 @@ gcd <- function(x, y) {
   ifelse(r, gcd(y, r), y)
 }
 
-vec_proxy_compare.vctrs_rational <- function(x) {
+vec_proxy_compare.vctrs_rational <- function(x, ...) {
   field(x, "n") / field(x, "d")
 }
 
@@ -40,14 +44,12 @@ rational_methods <- list(
   vec_ptype_abbr.vctrs_rational = function(x, ...) "rtnl",
   vec_ptype_full.vctrs_rational = function(x, ...) "rational",
 
-  vec_ptype2.vctrs_rational = function(x, y, ...) UseMethod("vec_ptype2.vctrs_rational", y),
-  vec_ptype2.vctrs_rational.default = function(x, y, ...) vec_default_ptype2(x, y, ...),
+  vec_ptype2.vctrs_rational = function(x, y, ...) UseMethod("vec_ptype2.vctrs_rational"),
   vec_ptype2.vctrs_rational.vctrs_rational = function(x, y, ...) new_rational(),
   vec_ptype2.vctrs_rational.integer = function(x, y, ...) new_rational(),
   vec_ptype2.integer.vctrs_rational = function(x, y, ...) new_rational(),
 
   vec_cast.vctrs_rational = function(x, to, ...) UseMethod("vec_cast.vctrs_rational"),
-  vec_cast.vctrs_rational.default = function(x, to, ...) vec_default_cast(x, to),
   vec_cast.vctrs_rational.vctrs_rational = function(x, to, ...) x,
   vec_cast.double.vctrs_rational = function(x, to, ...) field(x, "n") / field(x, "d"),
   vec_cast.vctrs_rational.integer = function(x, to, ...) rational(x, 1),
@@ -59,5 +61,3 @@ rational_methods <- list(
 local_rational_class <- function(frame = caller_env()) {
   local_methods(.frame = frame, !!!rational_methods)
 }
-
-env_bind(global_env(), !!!rational_methods)
