@@ -15,9 +15,9 @@ foobar <- new_ctor("vctrs_foobar")
 foobaz <- new_ctor("vctrs_foobaz")
 quux <- new_ctor("vctrs_quux")
 
-expect_foobar <- function(x) expect_is({{ x }}, "vctrs_foobar")
-expect_foobaz <- function(x) expect_is({{ x }}, "vctrs_foobaz")
-expect_quux <- function(x) expect_is({{ x }}, "vctrs_quux")
+expect_foobar <- function(x) expect_s3_class({{ x }}, "vctrs_foobar")
+expect_foobaz <- function(x) expect_s3_class({{ x }}, "vctrs_foobaz")
+expect_quux <- function(x) expect_s3_class({{ x }}, "vctrs_quux")
 
 with_c_foobar <- function(expr) {
   with_methods(
@@ -55,7 +55,7 @@ local_proxy <- function(frame = caller_env()) {
 new_proxy <- function(x) {
   structure(list(env(x = x)), class = "vctrs_proxy")
 }
-proxy_deref <- function(x) {
+proxy_deref <- function(x, ...) {
   x[[1]]$x
 }
 local_env_proxy <- function(frame = caller_env()) {
@@ -78,7 +78,7 @@ tibble <- function(...) {
 }
 
 local_foobar_proxy <- function(frame = caller_env()) {
-  local_methods(.frame = frame, vec_proxy.vctrs_foobar = identity)
+  local_methods(.frame = frame, vec_proxy.vctrs_foobar = function(x, ...) x)
 }
 
 subclass <- function(x) {
@@ -154,3 +154,9 @@ with_foobar_df_methods <- function(expr) {
   local_foobar_df_methods()
   expr
 }
+
+set_tibble <- function(x) {
+  base <- class(x)[-length(class(x))]
+  class(x) <- c(base, "tbl_df", "tbl", "data.frame")
+  x
+}
