@@ -905,71 +905,6 @@ test_that("duplicate needles match the same haystack locations", {
   expect_identical(x$haystack, c(1L, 3L, 2L, 1L, 3L, 2L))
 })
 
-test_that("`multiple` can error informatively", {
-  expect_snapshot({
-    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error")))
-    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo")))
-    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", error_call = call("fn"))))
-    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", haystack_arg = "bar")))
-  })
-})
-
-test_that("`multiple` can warn informatively", {
-  expect_snapshot({
-    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning")))
-    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo")))
-    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", error_call = call("fn"))))
-    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", haystack_arg = "bar")))
-  })
-})
-
-test_that("warning falls back to 'all'", {
-  expect_warning(
-    result <- vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "warning"),
-    class = "vctrs_warning_matches_multiple"
-  )
-
-  expect_identical(
-    result,
-    vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "all")
-  )
-})
-
-test_that("errors on multiple matches that come from different nesting containers", {
-  df <- data_frame(x = 0, y = 0)
-  df2 <- data_frame(x = 1:2, y = 2:1)
-
-  expect_error(
-    vec_locate_matches(df, df2, condition = c("<=", "<="), multiple = "error"),
-    "multiple matches"
-  )
-})
-
-test_that("errors when a match from a different nesting container is processed early on", {
-  # Row 1 has 2 matches
-  # Row 2 has 0 matches
-  needles <- data_frame(
-    a = c(1, 8),
-    b = c(2, 9)
-  )
-
-  # Rows 1 and 2 end up in different nesting containers
-  haystack <- data_frame(
-    a = c(5, 6),
-    b = c(7, 6)
-  )
-
-  # needles[1,] records the haystack[1,] match first, which is in the 1st
-  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
-  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
-  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
-  # multiple matches)
-  expect_error(
-    vec_locate_matches(needles, haystack, condition = "<", multiple = "error"),
-    "multiple matches"
-  )
-})
-
 test_that("correctly gets all matches when they come from different nesting containers", {
   needles <- data_frame(
     a = c(1, 8),
@@ -1022,12 +957,6 @@ test_that("correctly gets first/last/any match when they come from different nes
   )
 })
 
-test_that("`multiple = 'error'` doesn't error errneously on the last observation", {
-  expect_error(res <- vec_locate_matches(1:2, 1:2, multiple = "error"), NA)
-  expect_identical(res$needles, 1:2)
-  expect_identical(res$haystack, 1:2)
-})
-
 test_that("`multiple` is validated", {
   expect_snapshot({
     (expect_error(vec_locate_matches(1, 2, multiple = 1.5)))
@@ -1038,6 +967,431 @@ test_that("`multiple` is validated", {
   })
 })
 
+# ------------------------------------------------------------------------------
+# vec_locate_matches() - `multiple` (deprecated)
+
+test_that("`multiple` can error informatively", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error")))
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo")))
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", error_call = call("fn"))))
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", haystack_arg = "bar")))
+  })
+})
+
+test_that("`multiple` can warn informatively", {
+  expect_snapshot({
+    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning")))
+    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo")))
+    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", error_call = call("fn"))))
+    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", haystack_arg = "bar")))
+  })
+})
+
+test_that("warning falls back to 'all'", {
+  expect_warning(
+    result <- vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "warning"),
+    class = "vctrs_warning_matches_multiple"
+  )
+
+  expect_identical(
+    result,
+    vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "all")
+  )
+})
+
+test_that("errors on multiple matches that come from different nesting containers", {
+  df <- data_frame(x = 0, y = 0)
+  df2 <- data_frame(x = 1:2, y = 2:1)
+
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(df, df2, condition = c("<=", "<="), multiple = "error")
+  })
+})
+
+test_that("errors when a match from a different nesting container is processed early on", {
+  # Row 1 has 2 matches
+  # Row 2 has 0 matches
+  needles <- data_frame(
+    a = c(1, 8),
+    b = c(2, 9)
+  )
+
+  # Rows 1 and 2 end up in different nesting containers
+  haystack <- data_frame(
+    a = c(5, 6),
+    b = c(7, 6)
+  )
+
+  # needles[1,] records the haystack[1,] match first, which is in the 1st
+  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
+  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
+  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
+  # multiple matches)
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(needles, haystack, condition = "<", multiple = "error")
+  })
+})
+
+test_that("`multiple = 'error'` doesn't error errneously on the last observation", {
+  expect_error(res <- vec_locate_matches(1:2, 1:2, multiple = "error"), NA)
+  expect_identical(res$needles, 1:2)
+  expect_identical(res$haystack, 1:2)
+})
+
+test_that("`multiple = 'error' / 'warning'` throw correctly when combined with `relationship`", {
+  x <- c(1, 2, 2)
+  y <- c(2, 1, 2)
+
+  # `multiple` error technically fires first
+  expect_snapshot({
+    (expect_error(vec_locate_matches(x, y, relationship = "one-to-one", multiple = "error")))
+  })
+
+  # Works when warning is also requested
+  expect_snapshot({
+    (expect_error(vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "error")))
+  })
+  # Both warnings are thrown if applicable
+  expect_snapshot({
+    vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "warning")
+  })
+  # Both warning and error are thrown if applicable
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(x, y, relationship = "one-to-one", multiple = "warning")
+  })
+
+  x <- c(1, 2)
+  y <- c(2, 1, 2)
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "error")))
+  })
+  # Only `multiple` warning is applicable here
+  expect_snapshot({
+    vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "warning")
+  })
+})
+
+# ------------------------------------------------------------------------------
+# vec_locate_matches() - `relationship`
+
+test_that("`relationship` handles one-to-one case", {
+  # No error
+  expect_identical(
+    vec_locate_matches(1:2, 2:1, relationship = "one-to-one"),
+    vec_locate_matches(1:2, 2:1)
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    vec_locate_matches(1:2, 3:4, relationship = "one-to-one"),
+    vec_locate_matches(1:2, 3:4)
+  )
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(c(2, 1), c(1, 1), relationship = "one-to-one")))
+    (expect_error(vec_locate_matches(c(1, 1), c(1, 2), relationship = "one-to-one")))
+  })
+})
+
+test_that("`relationship` handles one-to-many case", {
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "one-to-many"),
+    vec_locate_matches(c(1, 2), c(1, 2, 2))
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    vec_locate_matches(1:2, 3:4, relationship = "one-to-many"),
+    vec_locate_matches(1:2, 3:4)
+  )
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(c(1, 2, 2), c(2, 1), relationship = "one-to-many")))
+  })
+})
+
+test_that("`relationship` handles many-to-one case", {
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "many-to-one"),
+    vec_locate_matches(c(1, 2, 2), c(1, 2))
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    vec_locate_matches(1:2, 3:4, relationship = "many-to-one"),
+    vec_locate_matches(1:2, 3:4)
+  )
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "many-to-one")))
+  })
+})
+
+test_that("`relationship` handles many-to-many case", {
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "many-to-many"),
+    vec_locate_matches(c(1, 2, 2), c(1, 2))
+  )
+
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "many-to-many"),
+    vec_locate_matches(c(1, 2), c(1, 2, 2))
+  )
+
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 1, 2), c(1, 2, 2), relationship = "many-to-many"),
+    vec_locate_matches(c(1, 1, 2), c(1, 2, 2))
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    vec_locate_matches(1:2, 3:4, relationship = "many-to-many"),
+    vec_locate_matches(1:2, 3:4)
+  )
+})
+
+test_that("`relationship` handles warn-many-to-many case", {
+  # No warning
+  expect_identical(
+    expect_silent(
+      vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "warn-many-to-many")
+    ),
+    vec_locate_matches(c(1, 2, 2), c(1, 2))
+  )
+
+  # No warning
+  expect_identical(
+    expect_silent(
+      vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "warn-many-to-many")
+    ),
+    vec_locate_matches(c(1, 2), c(1, 2, 2))
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    expect_silent(
+      vec_locate_matches(1:2, 3:4, relationship = "warn-many-to-many")
+    ),
+    vec_locate_matches(1:2, 3:4)
+  )
+
+  # Specifically designed to ensure we test both:
+  # - Finding multiple `needles` matches before multiple `haystack` matches
+  # - Finding multiple `haystack` matches before multiple `needles` matches
+  expect_snapshot({
+    (expect_warning(vec_locate_matches(c(1, 2, 1), c(1, 2, 2), relationship = "warn-many-to-many")))
+    (expect_warning(vec_locate_matches(c(1, 1, 2), c(2, 2, 1), relationship = "warn-many-to-many")))
+  })
+})
+
+test_that("`relationship` considers `incomplete` matches as possible multiple matches", {
+  x <- c(1, NA, NaN)
+  y <- c(NA, 1)
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(x, y, relationship = "one-to-many")))
+  })
+
+  # No error
+  expect_identical(
+    vec_locate_matches(x, y, relationship = "one-to-many", incomplete = NA),
+    vec_locate_matches(x, y, incomplete = NA)
+  )
+
+  # No error
+  expect_identical(
+    vec_locate_matches(x, y, relationship = "one-to-many", nan_distinct = TRUE),
+    vec_locate_matches(x, y, nan_distinct = TRUE)
+  )
+})
+
+test_that("`relationship` errors on multiple matches that come from different nesting containers", {
+  df <- data_frame(x = 0, y = 0)
+  df2 <- data_frame(x = 1:2, y = 2:1)
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(df, df2, condition = c("<=", "<="), relationship = "many-to-one")))
+  })
+})
+
+test_that("`relationship` errors when a match from a different nesting container is processed early on", {
+  # Row 1 has 2 matches
+  # Row 2 has 0 matches
+  needles <- data_frame(
+    a = c(1, 8),
+    b = c(2, 9)
+  )
+
+  # Rows 1 and 2 end up in different nesting containers
+  haystack <- data_frame(
+    a = c(5, 6),
+    b = c(7, 6)
+  )
+
+  # needles[1,] records the haystack[1,] match first, which is in the 1st
+  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
+  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
+  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
+  # multiple matches)
+  expect_snapshot({
+    (expect_error(vec_locate_matches(needles, haystack, condition = "<", relationship = "many-to-one")))
+  })
+})
+
+test_that("`relationship` doesn't error errneously on the last observation", {
+  expect_error(res <- vec_locate_matches(1:2, 1:2, relationship = "many-to-one"), NA)
+  expect_identical(res$needles, 1:2)
+  expect_identical(res$haystack, 1:2)
+})
+
+test_that("`relationship` doesn't error if `multiple` removes multiple matches", {
+  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "any", relationship = "one-to-one")
+  expect_identical(out$needles, c(1L, 2L))
+  expect_identical(out$haystack, c(1L, NA))
+
+  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "first", relationship = "one-to-one")
+  expect_identical(out$needles, c(1L, 2L))
+  expect_identical(out$haystack, c(1L, NA))
+
+  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "last", relationship = "one-to-one")
+  expect_identical(out$needles, c(1L, 2L))
+  expect_identical(out$haystack, c(2L, NA))
+})
+
+test_that("`relationship` can still detect problematic `haystack` relationships when `multiple = first/last` are used", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(c(3, 1, 1), c(2, 1, 3, 3), multiple = "first", relationship = "one-to-one")))
+    (expect_error(vec_locate_matches(c(3, 1, 1), c(2, 1, 3, 3), multiple = "first", relationship = "one-to-many")))
+  })
+})
+
+test_that("`relationship` and `remaining` work properly together", {
+  expect_snapshot({
+    out <- vec_locate_matches(
+      c(1, 2, 2),
+      c(2, 3, 1, 1, 4),
+      relationship = "warn-many-to-many",
+      remaining = NA_integer_
+    )
+  })
+  expect_identical(out$needles, c(1L, 1L, 2L, 3L, NA, NA))
+  expect_identical(out$haystack, c(3L, 4L, 1L, 1L, 2L, 5L))
+})
+
+test_that("`relationship` errors if `condition` creates multiple matches", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1, c(1, 2), condition = "<=", relationship = "many-to-one")))
+  })
+})
+
+test_that("`relationship` doesn't error if `filter` removes multiple matches", {
+  out <- vec_locate_matches(1, c(1, 2), condition = "<=", filter = "min", relationship = "many-to-one")
+  expect_identical(out$needles, 1L)
+  expect_identical(out$haystack, 1L)
+
+  out <- vec_locate_matches(1, c(1, 2), condition = "<=", filter = "max", relationship = "many-to-one")
+  expect_identical(out$needles, 1L)
+  expect_identical(out$haystack, 2L)
+})
+
+test_that("`relationship` still errors if `filter` hasn't removed all multiple matches", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1, c(1, 2, 1), condition = "<=", filter = "min", relationship = "many-to-one")))
+  })
+
+  # But not here
+  out <- vec_locate_matches(c(1, 1), c(1, 2, 1), condition = "<=", filter = "max", relationship = "many-to-one")
+  expect_identical(out$needles, c(1L, 2L))
+  expect_identical(out$haystack, c(2L, 2L))
+})
+
+test_that("`relationship` errors when we have >1 size 1 matches across containers (tidyverse/dplyr#6835)", {
+  # Carefully designed to ensure we get 2 nested containment groups that split
+  # up the rows of `y`, but each of the nested containment groups contain exactly
+  # 1 match, so `size_match` in `expand_compact_indices()` won't ever be >1
+  x <- data_frame(a = 1L, b = 5L)
+  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L))
+
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(
+      x,
+      y,
+      condition = c("<=", ">="),
+      filter = c("none", "none"),
+      relationship = "one-to-one"
+    )
+  })
+})
+
+test_that("`relationship` doesn't error when the first match from a different container gets filtered out (tidyverse/dplyr#6835)", {
+  # Carefully designed to ensure we get 2 nested containment groups that split
+  # up the rows of `y`. Row 1 (processed first) doesn't hold the minimum `b`
+  # value, so it gets filtered out. Row 2 is in the "extra" matches section
+  # but is actually the first (and only) real match, so we don't want to error
+  # on it.
+  x <- data_frame(a = 1L, b = 5L)
+  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L))
+
+  out <- vec_locate_matches(
+    x,
+    y,
+    condition = c("<=", ">="),
+    filter = c("none", "min"),
+    relationship = "one-to-one"
+  )
+  expect_identical(out$needles, 1L)
+  expect_identical(out$haystack, 2L)
+
+  # Similar to the above example, but with a `max` filter. Row 1 doesn't hold
+  # the max `c` value so it is filtered out even though it is a `>=` match.
+  x <- data_frame(a = 1L, b = 5L, c = 3L)
+  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L), c = c(1L, 2L))
+
+  out <- vec_locate_matches(
+    x,
+    y,
+    condition = c("<=", ">=", ">="),
+    filter = c("none", "none", "max"),
+    relationship = "one-to-one"
+  )
+  expect_identical(out$needles, 1L)
+  expect_identical(out$haystack, 2L)
+})
+
+test_that("`relationship` errors respect argument tags and error call", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), relationship = "one-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+    (expect_error(vec_locate_matches(c(1L, 1L), 1L, relationship = "one-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+    (expect_error(vec_locate_matches(c(1L, 1L), 1L, relationship = "one-to-many", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), relationship = "many-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+  })
+})
+
+test_that("`relationship` warnings respect argument tags and error call", {
+  expect_snapshot({
+    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", needles_arg = "foo", error_call = call("fn"))))
+    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", haystack_arg = "bar", error_call = call("fn"))))
+  })
+})
+
+test_that("`relationship` is validated", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1, 2, relationship = 1.5)))
+    (expect_error(vec_locate_matches(1, 2, relationship = c("one-to-one", "one-to-many"))))
+    (expect_error(vec_locate_matches(1, 2, relationship = "x")))
+    # Uses internal error
+    (expect_error(vec_locate_matches(1, 2, relationship = "x", error_call = call("fn"))))
+  })
+})
+
 # ------------------------------------------------------------------------------
 # vec_locate_matches() - `no_match`
 
@@ -1437,7 +1791,7 @@ test_that("potential overflow on large output size is caught informatively", {
   # intermediate `r_ssize` will be too large
   skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")
 
-  expect_snapshot(transform = scrub_internal_error_line_number, {
+  expect_snapshot({
     (expect_error(vec_locate_matches(1:1e7, 1:1e7, condition = ">=")))
   })
 })
