@@ -1,3 +1,4 @@
+local_name_repair_quiet()
 
 # vec_names() ---------------------------------------------------------
 
@@ -50,6 +51,10 @@ test_that("vec_names2() repairs names before invoking repair function", {
   expect_identical(vec_names2(x, repair = identity), c("", ""))
 })
 
+test_that("vec_names2() result is correct for *_quiet repair", {
+  expect_identical(vec_names2(1:2, repair = "unique"), vec_names2(1:2, repair = "unique_quiet"))
+  expect_identical(vec_names2(1:2, repair = "universal"), vec_names2(1:2, repair = "universal_quiet"))
+})
 
 # vec_as_names() -----------------------------------------------------------
 
@@ -58,8 +63,10 @@ test_that("vec_as_names() requires character vector", {
 })
 
 test_that("vec_as_names() validates `repair`", {
-  expect_error(vec_as_names("x", repair = "foo"), "can't be \"foo\"")
-  expect_error(vec_as_names(1, repair = 1), "string or a function")
+  expect_snapshot({
+    (expect_error(my_vec_as_names("x", my_repair = "foo"), "can't be \"foo\""))
+    (expect_error(my_vec_as_names(1, my_repair = 1), "string or a function"))
+  })
 })
 
 test_that("vec_as_names() repairs names", {
@@ -70,11 +77,24 @@ test_that("vec_as_names() repairs names", {
 })
 
 test_that("vec_as_names() checks unique names", {
-  expect_error(vec_as_names(chr(NA), repair = "check_unique"))
-  expect_error(vec_as_names(chr(""), repair = "check_unique"))
-  expect_error(vec_as_names(chr("a", "a"), repair = "check_unique"))
-  expect_error(vec_as_names(chr("..1"), repair = "check_unique"))
-  expect_error(vec_as_names(chr("..."), repair = "check_unique"))
+  expect_snapshot({
+    (expect_error(my_vec_as_names(chr(NA), my_repair = "check_unique")))
+    (expect_error(my_vec_as_names(chr(""), my_repair = "check_unique")))
+    (expect_error(my_vec_as_names(chr("a", "a"), my_repair = "check_unique")))
+    (expect_error(my_vec_as_names(chr("..1"), my_repair = "check_unique")))
+    (expect_error(my_vec_as_names(chr("..."), my_repair = "check_unique")))
+  })
+})
+
+test_that("vec_as_names() result is correct for *_quiet repair", {
+  expect_identical(
+    vec_as_names(chr("_foo", "_bar"), repair = "unique"),
+    vec_as_names(chr("_foo", "_bar"), repair = "unique_quiet")
+  )
+  expect_identical(
+    vec_as_names(chr("_foo", "_bar"), repair = "universal"),
+    vec_as_names(chr("_foo", "_bar"), repair = "universal_quiet")
+  )
 })
 
 test_that("vec_as_names() keeps the names of a named vector", {
@@ -102,7 +122,7 @@ test_that("vec_as_names() accepts and checks repair function", {
     ~ rep_along(.x, local_obj)
   })
   expect_identical(vec_as_names(c("", ""), repair = f), c("foo", "foo"))
-  expect_error(vec_as_names(c("", ""), repair = function(nms) "foo"), "length 1 instead of length 2")
+  expect_snapshot(error = TRUE, my_vec_as_names(c("", ""), my_repair = function(nms) "foo"))
 })
 
 test_that("vec_as_names() repairs names before invoking repair function", {
@@ -110,7 +130,9 @@ test_that("vec_as_names() repairs names before invoking repair function", {
 })
 
 test_that("vec_as_names() is noisy by default", {
-  verify_output(test_path("output", "test-vec-as-names.txt"), {
+  local_name_repair_verbose()
+
+  expect_snapshot({
     # Noisy name repair
     vec_as_names(c("x", "x"), repair = "unique")
 
@@ -118,22 +140,40 @@ test_that("vec_as_names() is noisy by default", {
     vec_as_names(c("x", "x"), repair = "unique", quiet = TRUE)
 
     # Hint at repair argument, if known
-    vec_as_names(c("x", "x"), repair = "check_unique", repair_arg = "repair")
+    (expect_error(
+      my_vec_as_names(c("x", "x"), my_repair = "check_unique")
+    ))
+
+    # request quiet via name repair string, don't specify `quiet`
+    vec_as_names(c("1", "1"), repair = "unique_quiet")
+    vec_as_names(c("1", "1"), repair = "universal_quiet")
+
+    # request quiet via name repair string, specify `quiet` = TRUE
+    vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = TRUE)
+    vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = TRUE)
+
+    # request quiet via name repair string, specify `quiet` = FALSE
+    vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = FALSE)
+    vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = FALSE)
   })
 })
 
 test_that("validate_minimal_names() checks names", {
-  expect_error(validate_minimal_names(1), "must return a character vector")
-  expect_error(validate_minimal_names(NULL), "can't return `NULL`")
-  expect_error(validate_minimal_names(chr(NA)), "can't return `NA` values")
+  expect_snapshot({
+    (expect_error(validate_minimal_names(1), "must return a character vector"))
+    (expect_error(validate_minimal_names(NULL), "can't return `NULL`"))
+    (expect_error(validate_minimal_names(chr(NA)), "can't return `NA` values"))
+  })
 })
 
 test_that("validate_unique() checks unique names", {
-  expect_error(validate_unique(chr(NA)), "`NA`")
-  expect_error(validate_unique(chr("")), class = "vctrs_error_names_cannot_be_empty")
-  expect_error(validate_unique(chr("a", "a")), class = "vctrs_error_names_must_be_unique")
-  expect_error(validate_unique(chr("..1")), class = "vctrs_error_names_cannot_be_dot_dot")
-  expect_error(validate_unique(chr("...")), class = "vctrs_error_names_cannot_be_dot_dot")
+  expect_snapshot({
+    (expect_error(validate_unique(chr(NA)), "`NA`"))
+    (expect_error(validate_unique(chr("")), class = "vctrs_error_names_cannot_be_empty"))
+    (expect_error(validate_unique(chr("a", "a")), class = "vctrs_error_names_must_be_unique"))
+    (expect_error(validate_unique(chr("..1")), class = "vctrs_error_names_cannot_be_dot_dot"))
+    (expect_error(validate_unique(chr("...")), class = "vctrs_error_names_cannot_be_dot_dot"))
+  })
 })
 
 test_that("vec_as_names_validate() validates repair arguments", {
@@ -157,6 +197,10 @@ test_that("vec_as_names() is quiet when function is supplied (#1018)", {
   )
 })
 
+test_that("vec_as_names() evaluates repair_arg lazily", {
+  expect_silent(vec_as_names(letters, repair_arg = print("oof")))
+})
+
 
 # vec_repair_names() -------------------------------------------------------
 
@@ -269,8 +313,10 @@ test_that("vec_set_names() can set NULL names", {
 })
 
 test_that("vec_set_names() errors with bad `names`", {
-  expect_error(vec_set_names(1, 1), "character vector, not a double")
-  expect_error(vec_set_names(1, c("x", "y")), "The size of `names`, 2")
+  expect_snapshot({
+    (expect_error(vec_set_names(1, 1), "character vector, not a double"))
+    (expect_error(vec_set_names(1, c("x", "y")), "The size of `names`, 2"))
+  })
 })
 
 test_that("vec_names() and vec_set_names() work with 1-dimensional arrays", {
@@ -343,12 +389,12 @@ test_that("as_unique_names() eliminates emptiness and duplication", {
   expect_identical(as_unique_names(x), c("...1", "x...2", "y", "x...4"))
 })
 
-test_that("solo empty or NA gets suffix", {
+test_that("as_unique_names(): solo empty or NA gets suffix", {
   expect_identical(as_unique_names(""), "...1")
   expect_identical(as_unique_names(NA_character_), "...1")
 })
 
-test_that("ellipsis treated like empty string", {
+test_that("as_unique_names() treats ellipsis like empty string", {
   expect_identical(as_unique_names("..."), as_unique_names(""))
 })
 
@@ -424,8 +470,10 @@ test_that("unique-ification has an 'algebraic'-y property", {
 })
 
 test_that("unique_names() and as_unique_names() are verbose or silent", {
-  expect_message(unique_names(1:2), "-> ...1", fixed = TRUE)
-  expect_message(as_unique_names(c("", "")), "-> ...1", fixed = TRUE)
+  local_name_repair_verbose()
+
+  expect_snapshot(unique_names(1:2))
+  expect_snapshot(as_unique_names(c("", "")))
 
   expect_message(regexp = NA, unique_names(1:2, quiet = TRUE))
   expect_message(regexp = NA, as_unique_names(c("", ""), quiet = TRUE))
@@ -458,12 +506,12 @@ test_that("dupes get a suffix", {
   )
 })
 
-test_that("solo empty or NA gets suffix", {
+test_that("as_universal_names(): solo empty or NA gets suffix", {
   expect_identical(as_universal_names(""), "...1")
   expect_identical(as_universal_names(NA_character_), "...1")
 })
 
-test_that("ellipsis treated like empty string", {
+test_that("as_universal_names() treats ellipsis like empty string", {
   expect_identical(as_universal_names("..."), as_universal_names(""))
 })
 
@@ -512,11 +560,8 @@ test_that("complicated inputs", {
 })
 
 test_that("message", {
-  expect_message(
-    as_universal_names(c("a b", "b c")),
-    "New names:\n* `a b` -> a.b\n* `b c` -> b.c\n",
-    fixed = TRUE
-  )
+  local_name_repair_verbose()
+  expect_snapshot(as_universal_names(c("a b", "b c")))
 })
 
 test_that("quiet", {
@@ -548,17 +593,9 @@ test_that("unnamed input gives uniquely named output", {
 })
 
 test_that("messages by default", {
-  expect_message(
-    vec_repair_names(set_names(1, "a:b"), "universal"),
-    "New names:\n* `a:b` -> a.b\n",
-    fixed = TRUE
-  )
-
-  expect_message(
-    vec_repair_names(set_names(1, "a:b"), ~ make.names(.)),
-    "New names:\n* `a:b` -> a.b\n",
-    fixed = TRUE
-  )
+  local_name_repair_verbose()
+  expect_snapshot(vec_repair_names(set_names(1, "a:b"), "universal"))
+  expect_snapshot(vec_repair_names(set_names(1, "a:b"), ~ make.names(.)))
 })
 
 test_that("quiet = TRUE", {
@@ -779,14 +816,18 @@ test_that("NULL name specs works with scalars", {
   expect_named(vec_c(foo = 1), "foo")
 
   expect_identical(apply_name_spec(NULL, "foo", chr(), 0L), chr())
+  expect_equal(vec_c(foo = dbl()), set_names(dbl(), ""))
   expect_named(vec_c(foo = set_names(dbl())), chr())
   expect_named(vec_c(foo = set_names(dbl()), bar = set_names(dbl())), chr())
 
   expect_error(apply_name_spec(NULL, "foo", c("a", "b")), "vector of length > 1")
-  expect_error(vec_c(foo = c(a = 1, b = 2)), "vector of length > 1")
-
   expect_error(apply_name_spec(NULL, "foo", NULL, 2L), "vector of length > 1")
-  expect_error(vec_c(foo = 1:2), "vector of length > 1")
+
+  expect_snapshot({
+    (expect_error(vec_c(foo = c(a = 1, b = 2)), "vector of length > 1"))
+    (expect_error(vec_c(foo = 1:2), "vector of length > 1"))
+    (expect_error(vec_c(x = c(xx = 1)), "named vector"))
+  })
 })
 
 test_that("function name spec is applied", {
@@ -840,7 +881,7 @@ test_that("r_chr_paste_prefix() works", {
   nms <- c("foo", "bar")
 
   expect_equal(
-    .Call(vctrs_chr_paste_prefix, nms, "baz", "."),
+    .Call(ffi_chr_paste_prefix, nms, "baz", "."),
     c("baz.foo", "baz.bar")
   )
 
@@ -848,7 +889,14 @@ test_that("r_chr_paste_prefix() works", {
   long_prefix <- strrep("a", 5000)
 
   expect_equal(
-    .Call(vctrs_chr_paste_prefix, nms, long_prefix, "."),
+    .Call(ffi_chr_paste_prefix, nms, long_prefix, "."),
     paste0(long_prefix, ".", nms)
   )
 })
+
+test_that("vec_as_names() uses internal error if `repair_arg` is not supplied", {
+  expect_snapshot({
+    (expect_error(vec_as_names("", repair = "foobar", call = quote(tilt()))))
+    (expect_error(vec_as_names("", repair = env(), call = quote(tilt()))))
+  })
+})
