@@ -40,39 +40,18 @@ test_that("is_list_of as expected", {
 
 test_that("print method gives human friendly output", {
   skip_on_cran() # Depends on tibble
-
-  # FIXME: Disable crayon until we switch to testthat 3e
-  local_options(crayon.enabled = FALSE)
-
-  x <- list_of(1, 2:3)
-
-  expect_known_output({
-      print(x)
-      cat("\n")
-      print(tibble::tibble(x))
-    },
-    file = test_path("test-list_of-print.txt")
-  )
+  expect_snapshot(list_of(1, 2:3))
+  expect_snapshot(tibble::tibble(x = list_of(1, 2:3)))
 })
 
 test_that("str method is reasonably correct", {
   x <- list_of(1, 2:3)
 
-  expect_known_output({
-      str(x)
-      cat("\n")
-      str(list(list(x, y = 2:1)))
-    },
-    file = test_path("test-list_of-str.txt")
-  )
+  expect_snapshot(str(x))
+  expect_snapshot(str(list(list(x, y = 2:1))))
 
-  expect_known_output({
-      str(x[0])
-      cat("\n")
-      str(list(list(x, y = 2:1)))
-    },
-    file = test_path("test-list_of-str-empty.txt")
-  )
+  expect_snapshot(str(x[0]))
+  expect_snapshot(str(list(list(x[0], y = 2:1))))
 })
 
 test_that("constructor requires list input", {
@@ -84,6 +63,15 @@ test_that("constructor requires size 0 ptype", {
   expect_error(new_list_of(ptype = 1), "must have size 0")
 })
 
+test_that("can combine a mix of named and unnamed list-ofs (#784)", {
+  a <- new_list_of(list(x = 1L), ptype = integer())
+  b <- new_list_of(list(2L), ptype = integer())
+
+  expect <- new_list_of(list(x = 1L, 2L), ptype = integer())
+
+  expect_identical(vec_c(a, b), expect)
+})
+
 # Subsetting --------------------------------------------------------------
 
 test_that("[ preserves type", {
@@ -131,7 +119,7 @@ test_that("assingment can increase size of vector", {
 # Type system -------------------------------------------------------------
 
 test_that("list_of() are vectors", {
-  expect_true(vec_is_vector(list_of(1)))
+  expect_true(obj_is_vector(list_of(1)))
   expect_true(vec_is(list_of(1)))
 })
 
@@ -145,12 +133,9 @@ test_that("list coercions are symmetric and unchanging", {
   mat <- maxtype_mat(types)
 
   expect_true(isSymmetric(mat))
-  expect_known_output(
-    mat,
-    test_path("test-list_of-type.txt"),
-    print = TRUE,
-    width = 200
-  )
+
+  local_options(width = 200)
+  expect_snapshot(print(mat))
 })
 
 test_that("max<list_of<a>, list_of<b>> is list_of<max<a, b>>", {
@@ -161,17 +146,50 @@ test_that("max<list_of<a>, list_of<b>> is list_of<max<a, b>>", {
   expect_equal(vec_ptype_common(r_int, r_dbl), r_int)
 })
 
+test_that("can cast to self type", {
+  x <- list_of(1)
+  expect_identical(vec_cast(x, x), x)
+})
+
+test_that("can cast between different list_of types", {
+  x <- list_of(1, 2)
+  to <- list_of(.ptype = integer())
+  expect_identical(vec_cast(x, to), list_of(1L, 2L))
+})
+
+test_that("list_of casting retains outer names", {
+  x <- list_of(x = 1, 2, z = 3)
+  to <- list_of(.ptype = integer())
+  expect_named(vec_cast(x, to), c("x", "", "z"))
+})
+
 test_that("safe casts work as expected", {
   x <- list_of(1)
   expect_equal(vec_cast(NULL, x), NULL)
   expect_equal(vec_cast(NA, x), list_of(NULL, .ptype = double()))
 
+  expect_identical(vec_cast(list(1), x), list_of(1))
+  expect_identical(vec_cast(list(TRUE), x), list_of(1))
+  expect_identical(vec_cast(x, list()), list(1))
+  expect_identical(vec_cast(x, list()), list(1))
+
+  expect_error(
+    vec_cast(list_of(1), list_of("")),
+    class = "vctrs_error_incompatible_type"
+  )
+
   # These used to be allowed
   expect_error(vec_cast(1L, x), class = "vctrs_error_incompatible_type")
   expect_error(vec_cast(1, x), class = "vctrs_error_incompatible_type")
-  expect_error(vec_cast(list(1), x), class = "vctrs_error_incompatible_type")
-  expect_error(vec_cast(list(TRUE), x), class = "vctrs_error_incompatible_type")
-  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
+})
+
+test_that("error call is passed to inner cast methods", {
+  fn1 <- function() vec_cast(list_of(1), list_of(""))
+  fn2 <- function() vec_cast(list(1), list_of(""))
+  expect_snapshot({
+    (expect_error(fn1()))
+    (expect_error(fn2()))
+  })
 })
 
 test_that("lossy casts generate warning (no longer the case)", {
@@ -186,14 +204,6 @@ test_that("invalid casts generate error", {
   expect_error(vec_cast(factor("a"), list_of(1)), class = "vctrs_error_incompatible_type")
 })
 
-test_that("validation", {
-  expect_error(validate_list_of(list_of(1, 2, 3)), NA)
-  expect_error(
-    validate_list_of(new_list_of(list(factor("foo")), vec_ptype(factor("bar")))),
-    class = "vctrs_error_cast_lossy"
-  )
-})
-
 test_that("list_of() has as.character() method (tidyverse/tidyr#654)", {
   exp <- rep(paste0("<", vec_ptype_abbr(mtcars), ">"), 2)
   expect_identical(as.character(list_of(mtcars, mtcars)), exp)
@@ -204,3 +214,31 @@ test_that("vec_ptype2(<list_of<>>, NA) is symmetric (#687)", {
   expect_identical(vec_ptype2(lof, NA), vec_ptype(lof))
   expect_identical(vec_ptype2(NA, lof), vec_ptype(lof))
 })
+
+test_that("list_of() coerces to list() and list_of() (#1701)", {
+  expect_equal(vec_ptype_common(list_of(1), list()), list())
+  expect_equal(vec_cast_common(list_of(1), list()), list(list(1), list()))
+
+  expect_equal(vec_ptype_common(list_of(1), list("")), list())
+  expect_equal(vec_cast_common(list_of(1), list("")), list(list(1), list("")))
+
+  expect_equal(
+    vec_ptype_common(list_of(1), list_of("")),
+    list()
+  )
+  expect_equal(
+    vec_ptype_common(list_of(1), list(), list_of("")),
+    list()
+  )
+})
+
+test_that("can concatenate list and list-of (#1161)", {
+  expect_equal(
+    vec_c(list(1), list_of(2)),
+    list(1, 2)
+  )
+  expect_equal(
+    vec_c(list(""), list_of(2)),
+    list("", 2)
+  )
+})
